@@ -31,9 +31,16 @@ extern int ssl_no_fail;
 extern char *openssl_pem;
 extern char *ssl_certs_dir;
 extern char *enc_str;
+extern int vencrypt_mode;
+extern int vencrypt_kx;
+extern int vencrypt_enable_plain_login;
+extern int anontls_mode;
+extern int create_fresh_dhparams;
+extern char *dhparams_file;
 extern int https_port_num;
 extern int https_port_redir;
 extern char *ssl_verify;
+extern char *ssl_crl;
 extern int ssl_initialized;
 extern int ssl_timeout_secs;
 extern char *ssh_str;
@@ -85,6 +92,7 @@ extern char **passwd_list;
 extern int begin_viewonly;
 extern int inetd;
 extern int tightfilexfer; 
+extern int got_ultrafilexfer; 
 extern int first_conn_timeout;
 extern int ping_interval;
 extern int flash_cmap;
@@ -266,6 +274,7 @@ extern double slow_fb;
 extern double xrefresh;
 extern int wait_bog;
 extern int defer_update;
+extern int set_defer;
 extern int got_defer;
 extern int got_deferupdate;
 
