@@ -134,6 +134,7 @@ static void initialize_xevents(int reset) {
 		selwin = XCreateSimpleWindow(dpy, rootwin, 0, 0, 1, 1, 0, 0, 0);
 		X_UNLOCK;
 		did_xcreate_simple_window = 1;
+		if (0) rfbLog("selwin: 0x%lx\n", selwin);
 	}
 
 	if ((xrandr || xrandr_maybe) && !did_xrandr) {
@@ -1545,14 +1546,19 @@ static void try_local_chat_window(void) {
 		int d;
 		args[0] = "/bin/sh";
 		args[1] = "-c";
-		/* "ssvnc -cmd VNC://fd=0 -chatonly"; */
+		/* "ssvnc -cmd VNC://fd=0 -chatonly"; not working */
 		args[2] = cmd;
 		args[3] = NULL;
 
+		set_env("VNCVIEWER_PASSWORD", "moo");
+#if !NO_X11
+		if (dpy != NULL) {
+			set_env("DISPLAY", DisplayString(dpy));
+		}
+#endif
 		for (d = 3; d < 256; d++) {
 			close(d);
 		}
-		set_env("VNCVIEWER_PASSWORD", "moo");
 
 		execvp(args[0], args);
 		perror("exec");
@@ -1623,6 +1629,7 @@ void set_text_chat(rfbClientPtr cl, int len, char *txt) {
 		rfbCloseClient(cl);
 		return;
 	}
+#if LIBVNCSERVER_HAS_TEXTCHAT
 
 	if (chat_window && chat_window_client == NULL && ulen == rfbTextChatOpen) {
 		try_local_chat_window();
@@ -1657,6 +1664,7 @@ void set_text_chat(rfbClientPtr cl, int len, char *txt) {
 		/* not clear what is going on WRT close and finished... */
 		rfbSendTextChatMessage(cl, rfbTextChatFinished, "");
 	}
+#endif
 }
 
 int get_keyboard_led_state_hook(rfbScreenInfoPtr s) {
